/**********************************************************************************/
/*                                                                                */
/* File:   draw.cpp                                                               */
/* Author: bkenwright@xbdev.net                                                   */
/* URL:    www.xbdev.net                                                          */
/* Date:   21-12-2005 (xmas)                                                      */
/*                                                                                */
/**********************************************************************************/



#include <windows.h>				// Windows header files
#include <assert.h>					// assert(..)


#pragma comment(lib, "d3dx9.lib")	// directx library files
#pragma comment(lib, "d3d9.lib")

#include <d3d9.h>					// directx header files
#include <d3dx9.h>
#include <d3dx9mesh.h>  

#include <stdio.h>					//sprintf(..)

#include "dxdbg.h"

#include "common.h"					// SafeRelease(..)

#include "bsp.h"


#define	 BUILD_VERSION				"0.0.01a"

#define  SZ_FILE_BSP				"test_level.bsp"		// Quake 3 BSP Level File

		


LPDIRECT3D9							g_pD3D					= NULL;
LPDIRECT3DDEVICE9					g_pd3dDevice			= NULL;

HWND								g_hwnd					= 0;

static float						g_fSpinX				= 0.0f;
static float						g_fSpinY				= 90.0f;
static float						g_fZoom					= -200.0f;

static bool							g_bWire					= true;


//--------------------------------------------------------------------------------//
//        Creation/Destruction of our DirectX Applications                        //
//--------------------------------------------------------------------------------//

bool Create(HWND hWnd)
{
	g_hwnd = hWnd;
    //First of all, create the main D3D object. If it is created successfully 
	//we should get a pointer to an IDirect3D8 interface.
    g_pD3D = Direct3DCreate9(D3D_SDK_VERSION);
	if( g_pD3D==NULL )
	{
		ErrMsg("Error Creating DirectX Object Instance");
		return false;
	}

    //Get the current display mode
    D3DDISPLAYMODE d3ddm;
	HRESULT hr = 
    g_pD3D->GetAdapterDisplayMode(D3DADAPTER_DEFAULT, &d3ddm);
	if( hr!=S_OK )
	{
		ErrMsg("Failed Getting Display Mode");
		return false;
	}

    //Create a structure to hold the settings for our device
    D3DPRESENT_PARAMETERS d3dpp; 
    ZeroMemory(&d3dpp, sizeof(d3dpp));
    //Fill the structure. 
    //We want our program to be windowed, and set the back buffer to a format
    //that matches our current display mode
    d3dpp.Windowed				 = TRUE;
    d3dpp.SwapEffect			 = D3DSWAPEFFECT_DISCARD;
    d3dpp.BackBufferFormat		 = d3ddm.Format;
    //For depth buffering (e.g.) the z-buffer
    d3dpp.BackBufferCount		 = 1;
    d3dpp.AutoDepthStencilFormat = D3DFMT_D16;
    d3dpp.EnableAutoDepthStencil = TRUE;
    
	//Create a Direct3D device
	hr =														// Error Checking
    g_pD3D->CreateDevice(D3DADAPTER_DEFAULT,					// UINT Adapter
						 D3DDEVTYPE_HAL,						// D3DDEVTYPE DeviceType
						 hWnd,									// HWND hFocusWindow
						 D3DCREATE_SOFTWARE_VERTEXPROCESSING,	// DWORD BehaviorFlags
						 &d3dpp,								// D3DPRESENT_PARAMETERS *
						 &g_pd3dDevice);						// IDirect3DDevice9 *
	if( hr!=S_OK )
	{
		ErrMsg("Failed Creating Rendering Device");
		return false;
	}


    //Turn on z-buffering
    g_pd3dDevice->SetRenderState(D3DRS_ZENABLE, D3DZB_TRUE);

	//Set our culling mode to Counter Clock Wise
	g_pd3dDevice->SetRenderState(D3DRS_CULLMODE, D3DCULL_CCW);



	return true;
}//End Create(..)


bool Release()
{
	SafeRelease( g_pd3dDevice );

	SafeRelease( g_pD3D );

	return true;
}//End Release()

//--------------------------------------------------------------------------------//

// Capture mouse input, and use it to make our demo more interactive
void MouseActions(int x,     // x mouse position
				  int y,     // y mouse position
				  int lmb,   // 0 None, 1 Button Down, 2 Button Released
				  int rmb,   // 0 None, 1 Button Down, 2 Button Released
				  int mmb)   // 0 None, 1 Button Down, 2 Button Released
{
	static int oldy = x;
	static int oldx = y;

	static bool bSpin = false;
	static bool bZoom = false;

	int dx = oldx - x;
	int dy = oldy - y;

	oldx = x;
	oldy = y;

	if( lmb== 1 )
		bSpin = true;
	if( lmb==2 )
		bSpin = false;

	if( rmb==1 )
		bZoom = true;
	if( rmb==2 )
		bZoom = false;
		

	if( bSpin )
	{
		g_fSpinX+=0.01f*dx;
		g_fSpinY+=0.01f*dy;
	}

	if( bZoom )
		g_fZoom+=0.5f*dy;
}



void KeyBoard()
{
	if( GetKeyState('W')		& 0x80 )  g_bWire = !g_bWire;
}



//-------------------------------------------------------------------------//

void SetMaterial()
{
      D3DMATERIAL9 matMaterial;
      D3DCOLORVALUE rgbaDiffuse  = {1.0, 1.0, 1.0, 0.0};  //RGBA
      D3DCOLORVALUE rgbaAmbient  = {1.0, 1.0, 1.0, 0.0};
      D3DCOLORVALUE rgbaSpecular = {0.0, 0.0, 0.0, 0.0};
      // emissive is for a glow effect
      D3DCOLORVALUE rgbaEmissive = {0.0, 0.0, 0.0, 0.0};

      matMaterial.Diffuse  = rgbaDiffuse;
      matMaterial.Ambient  = rgbaAmbient;
      matMaterial.Specular = rgbaSpecular;
      matMaterial.Emissive = rgbaEmissive;
      matMaterial.Power    = 100.0f;

      g_pd3dDevice->SetMaterial(&matMaterial);
}//End SetMaterial(..)

 

void SetLight()
{
	//Set a light up?
    D3DLIGHT9 d3dLight;

    //Initialize the light structure.
    ZeroMemory(&d3dLight, sizeof(D3DLIGHT9));

    //Set up a white point light at (0, 0, -10).
    d3dLight.Type			= D3DLIGHT_POINT;

    // Actual colour of the light. (e.g. white).
    d3dLight.Diffuse.r		= 1.0f;
    d3dLight.Diffuse.g		= 1.0f;
    d3dLight.Diffuse.b		= 1.0f;

    // Colour when no light hits the object.
    d3dLight.Ambient.r		= 0.2f;
    d3dLight.Ambient.g		= 0.2f;
    d3dLight.Ambient.b		= 0.2f;

    // Shiny effect.
    d3dLight.Specular.r		= 0.1f;
    d3dLight.Specular.g     = 0.1f;
    d3dLight.Specular.b     = 0.1f;

    d3dLight.Position.x     = 0.0f;
    d3dLight.Position.y     = 0.0f;
    d3dLight.Position.z     = -30.0f;

    d3dLight.Attenuation0	= 1.0f; 
    d3dLight.Attenuation1	= 0.0f; 
    d3dLight.Attenuation2	= 0.0f; 
    d3dLight.Range			= 100.0f;

    g_pd3dDevice->SetLight(0, &d3dLight);
    g_pd3dDevice->LightEnable(0,TRUE);

    //Turn on lighting
    g_pd3dDevice->SetRenderState(D3DRS_LIGHTING, TRUE);
    g_pd3dDevice->SetRenderState(D3DRS_AMBIENT, 
								 D3DCOLOR_XRGB(20,20,20));
}//End SetLight(..)



void SetCamera()
{
	D3DXVECTOR3 vCamPosition = D3DXVECTOR3( 0, 0, g_fZoom );

    // [1] D3DTS_VIEW
    D3DXMATRIX v;
    g_pd3dDevice->SetTransform(	D3DTS_VIEW, 
								D3DXMatrixLookAtLH(&v, 
								&vCamPosition, 
                                &D3DXVECTOR3(0,0,0), 
                                &D3DXVECTOR3(0,1,0)));  
    // [2] D3DTS_PROJECTION
    D3DXMATRIX p;
    g_pd3dDevice->SetTransform( D3DTS_PROJECTION, 
								D3DXMatrixPerspectiveFovLH( &p,  
								D3DX_PI/4,  
								1.0f,
                                1.0f,800.0f));

	// [3] 	D3DTS_WORLD
	D3DXMATRIX matRot, matScale;
	D3DXMatrixRotationYawPitchRoll( &matRot, 
									g_fSpinX, 
									g_fSpinY, 
									0);
	D3DXMatrixScaling(&matScale,
					  0.1f,		// x scale
					  0.1f,		// y scale
					  0.1f);	// z scale
	g_pd3dDevice->SetTransform( D3DTS_WORLD, 
								&(matRot*matScale));
}//End SetCamera(..)



////////////////////////////////////////////////////////////////////////////////////
// Display information to the user on the screen
void RenderInfo()
{
	// Using a static variable so we don't need a global
	static debugtxt txt(g_pd3dDevice);

	// Get the window size
	RECT rect;
	GetClientRect(g_hwnd, &rect);

	int yl = rect.bottom - 20;    // y left screen pos text
	int yr = yl;				  // y right screen pos text
	int xr = rect.right  - 150;	  // x right screen pos text

	char buf[200]; // large temp txt buffer
	sprintf(buf, "X-Rot:%.2f  Y-Rot:%.2f", g_fSpinX, g_fSpinY);
	txt.printf(buf, 10, yl ); yl-=20;

	sprintf(buf, "Z-Zoom:%.2f", g_fZoom);
	txt.printf(buf, 10, yl ); yl-=20;

	sprintf(buf, "Compiled: %s", __DATE__);
	txt.printf(buf, xr, yr ); yr-=20;

	sprintf(buf, "Build: %s", BUILD_VERSION);
	txt.printf(buf, xr, yr ); yr-=20;
}


////////////////////////////////////////////////////////////////////////////////////
//  
//  Render(..)
//
////////////////////////////////////////////////////////////////////////////////////
//
//  Our main render loop, which gets called over and over agian to do our
//  drawing...clears the screen, draws the data, then presents it..        
//
////////////////////////////////////////////////////////////////////////////////////
void Render()
{

	SetCamera();

	SetLight();

	SetMaterial();

	KeyBoard();

	if( g_bWire )
		g_pd3dDevice->SetRenderState( D3DRS_FILLMODE, D3DFILL_WIREFRAME );
	else
		g_pd3dDevice->SetRenderState( D3DRS_FILLMODE, D3DFILL_SOLID );


	g_pd3dDevice->Clear( 0, 
						 NULL, 
						 D3DCLEAR_TARGET | D3DCLEAR_ZBUFFER, 
						 0x88888800, 
						 1.0f, 
						 0 );
    g_pd3dDevice->BeginScene();
	


	static CBSP bsp(SZ_FILE_BSP);

	g_pd3dDevice->SetRenderState(D3DRS_CULLMODE, D3DCULL_NONE);
	bsp.Render(g_pd3dDevice);

	RenderInfo();

	TriDisplay(g_pd3dDevice, bsp.GetNumFaces() );
	FPSDisplay(g_pd3dDevice);
    g_pd3dDevice->EndScene();
	g_pd3dDevice->Present( NULL, NULL, NULL, NULL );

}//End Render()


